
/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <assert.h>

#include "initng.h"

#include "initng_global.h"

#include "initng_toolbox.h"
#include "initng_plugin_hook.h"
#include "initng_plugin.h"

/* deletes hook by hook type, and pointer */
int initng_del_hook2(e_ptype t, void *hook, const char *func, int line)
{
    s_call *current = g.call_db;
    s_call *next = NULL;

    assert(hook);
    D_("Delete a hooktype: %i\n", t);

    if (!g.call_db)
    {
        F_("There is no calls to delete!\n");
        return (FALSE);
    }

    if (g.call_db->ptype == t && g.call_db->c.hook == hook)
    {
        next = g.call_db->next;
        free(g.call_db);
        g.call_db = next;
        D_("This was the first hook, deleted.\n");
        return (TRUE);
    }
    D_("This was not the first hook.\n");

    while (current->next)
    {
        /*printf(" %i:(%i)  %i ? %i:(%i) \n", current->next->ptype, t , (int) current->next->c.hook, (int) current->next->c.fdh.call_module,(int) hook); */
        if (current->next->ptype == t && current->next->c.hook == hook)
        {
            next = current->next->next;
            free(current->next);
            current->next = next;
            D_("Hook type %i deleted.\n", t);
            return (TRUE);
        }
        current = current->next;
    }

    D_("Did not find hook type: %i to delete, added from %s line %i!!!!!\n",
       t, func, line);
    return (FALSE);
}


int initng_add_hook(e_ptype t, int order, void *hook, ...)
{
    /* TODO, this has to be cleaned up, we should scan va_start dynamically 
       and make one standard "int plugin_ext" or something for extended plugin data
       Has the added bonus that plugins can eventually use that as (char*) and add their own buffer of data?
     */
    va_list list;
    int fd;
    s_call *current = g.call_db;
    s_call *save = NULL;

    assert(hook);
    D_("Adding hook type %i\n", t);


    /* add the #1st call manually */
    if (!g.call_db)
    {
        g.call_db = initng_calloc(1, sizeof(s_call));
        g.call_db->next = NULL;
        if (t == FDWATCHERS)
        {
            /* Add us to FD watchers */
            va_start(list, hook);
            fd = va_arg(list, int);

            va_end(list);
            if (fd < 3)
            {
                F_("Filedescriptor must be >= 3.\n");
                return (FALSE);
            }
            g.call_db->ptype = t;
            g.call_db->c.fdh.call_module = hook;
            g.call_db->c.fdh.fds = fd;
            g.call_db->order = order;
            D_("ADDED FD %i\n", fd);
            return (TRUE);

        }

        g.call_db->ptype = t;
        g.call_db->c.hook = hook;
        g.call_db->order = order;
        D_("hook added.\n");
        return (TRUE);
    }

    /* this might have to be before the 1st one */
    if (order < g.call_db->order)
    {
        save = g.call_db;
        g.call_db = initng_calloc(1, sizeof(s_call));
        g.call_db->next = save;

        if (t == FDWATCHERS)
        {
            /* Add us to FD watchers */
            va_start(list, hook);
            fd = va_arg(list, int);

            va_end(list);
            if (fd < 3)
            {
                F_("Filedescriptor must be >= 3.\n");
                return (FALSE);
            }
            g.call_db->ptype = t;
            g.call_db->c.fdh.call_module = hook;
            g.call_db->c.fdh.fds = fd;
            g.call_db->order = order;
            D_("ADDED FD %i\n", fd);
            return (TRUE);

        }

        g.call_db->ptype = t;
        g.call_db->c.hook = hook;
        g.call_db->order = order;
        D_("hook added.\n");
        return (TRUE);
    }

    /* we will put it in in the right order */
    /* go forward to we find the pointers in this order */
    while (current->next && current->next->order < order)
        current = current->next;


    save = current->next;
    current->next = initng_calloc(1, sizeof(s_call));
    current->next->next = save;
    current = current->next;


    if (t == FDWATCHERS)
    {
        /* Add us to FD watchers */
        va_start(list, hook);
        fd = va_arg(list, int);

        va_end(list);
        if (fd < 3)
        {
            F_("Filedescriptor must be >= 3.\n");
            return (FALSE);
        }
        current->ptype = t;
        current->c.fdh.call_module = hook;
        current->c.fdh.fds = fd;
        current->order = order;
        D_("ADDED FD %i\n", fd);
        return (TRUE);

    }

    current->ptype = t;
    current->c.hook = hook;
    current->order = order;
    D_("hook added.\n");
    return (TRUE);
}
